﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDataModels.Models;
using FurnitureAssemblyContracts.BusinessLogicsContracts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FurnitureAssembly
{
    public partial class FormAddToShop : Form
    {

        private readonly List<ShopViewModel>? _listShops;
        private readonly List<FurnitureViewModel>? _listFurnitures;

        public int Id
        {
            get
            {
                return Convert.ToInt32(comboBoxShop.SelectedValue);
            }
            set
            {
                comboBoxShop.SelectedValue = value;
            }
        }
        public IShopModel? ShopModel
        {
            get
            {
                if (_listShops == null)
                {
                    return null;
                }
                foreach (var elem in _listShops)
                {
                    if (elem.Id == Id)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }

        public int FurnitureId
        {
            get
            {
                return Convert.ToInt32(comboBoxFurniture.SelectedValue);
            }
            set
            {
                comboBoxFurniture.SelectedValue = value;
            }
        }
        public IFurnitureModel? FurnitureModel
        {
            get
            {
                if (_listFurnitures == null)
                {
                    return null;
                }
                foreach (var elem in _listFurnitures)
                {
                    if (elem.Id == Id)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }

        public int Count
        {
            get { return Convert.ToInt32(textBoxCount.Text); }
            set
            { textBoxCount.Text = value.ToString(); }
        }
        public FormAddToShop(IShopLogic shopLogic, IfurnitureLogic furnitureLogic)
        {
            InitializeComponent();            
            _listShops = shopLogic.ReadList(null);
            _listFurnitures = furnitureLogic.ReadList(null);
            if (_listShops != null)
            {
                comboBoxShop.DisplayMember = "ShopName";
                comboBoxShop.ValueMember = "Id";
                comboBoxShop.DataSource = _listShops;
                comboBoxShop.SelectedItem = null;
            }
            if (_listFurnitures != null)
            {
                comboBoxFurniture.DisplayMember = "FurnitureName";
                comboBoxFurniture.ValueMember = "Id";
                comboBoxFurniture.DataSource = _listFurnitures;
                comboBoxFurniture.SelectedItem = null;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxFurniture.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Укажите количество поступившего изделия", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
