﻿using FurnitureAssemblyDataModels.Enums;
using FurnitureAssembly;
using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FurnitureAssemblyView
{
    public partial class MainForm : Form
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _orderLogic;
        private readonly IShopLogic _shopLogic;

        public MainForm(ILogger<MainForm> logger, IOrderLogic orderLogic, IShopLogic shopLogic)
        {
            InitializeComponent();
            _logger = logger;
            _orderLogic = orderLogic;
            _shopLogic = shopLogic;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            LoadData();
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка заказов");
            try
            {
                var list = _orderLogic.ReadList(null);

                if (list != null)
                {
                    dataGridView.DataSource = list;
                    dataGridView.Columns["FurnitureId"].Visible = false;
                }

                _logger.LogInformation("Загрузка заказов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заказов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void компонентыToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(ComponentsForm));
            if (service is ComponentsForm form)
            {
                form.ShowDialog();
            }
        }

        private void изделияToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FurnituresForm));

            if (service is FurnituresForm form)
            {
                form.ShowDialog();
            }
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(OrderForm));
            if (service is OrderForm form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private void buttonToWork_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
                try
                {
                    var operationResult = _orderLogic.TakeOrderInWork(new OrderBindingModel {
                        Id = id,
                        FurnitureId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["FurnitureId"].Value),
                        FurnitureName = dataGridView.SelectedRows[0].Cells["FurnitureName"].Value.ToString(),
                        Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                        Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
                        Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                        DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
                    });
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка передачи заказа в работу");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }

        }

        private void buttonReady_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'", id);
                try
                {
                    var operationResult = _orderLogic.FinishOrder(new OrderBindingModel
                    {
                        Id = id,
                        FurnitureId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["FurnitureId"].Value),
                        FurnitureName = dataGridView.SelectedRows[0].Cells["FurnitureName"].Value.ToString(),
                        Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                        Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
                        Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                        DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
                    });
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о готовности заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        private void buttonPut_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);
                try
                {
                    var operationResult = _orderLogic.DeliveryOrder(new
                   OrderBindingModel
                    {
                        Id = id,
                        FurnitureId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["FurnitureId"].Value),
                        FurnitureName = dataGridView.SelectedRows[0].Cells["FurnitureName"].Value.ToString(),
                        Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                        Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
                        Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                        DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
                    });
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    _logger.LogInformation("Заказ №{id} выдан", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о выдачи заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                   MessageBoxIcon.Error);
                }
            }

        }

        private void buttonRefresh_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void ShopToolStripMenuItem_Click(object sender, EventArgs e)
        {

            var service = Program.ServiceProvider?.GetService(typeof(FormShops));
            if (service is FormShops form)
            {
                form.ShowDialog();
            }
        }

        private void AddToShopToolStripMenuItem_Click(object sender, EventArgs e)
        {
                var service = Program.ServiceProvider?.GetService(typeof(FormAddToShop));
                if (service is FormAddToShop form)
                {
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.ShopModel == null || form.FurnitureModel == null)
                        {
                            return;
                        }
                        _logger.LogInformation("Добавление в магазин {ShopName} изделия: { FurnitureName}- { Count}", form.ShopModel.ShopName, form.FurnitureModel.FurnitureName, form.Count);

                        var modelShop = new ShopBindingModel
                        {
                            Id = form.Id,
                            ShopName = form.ShopModel.ShopName,
                            Address = form.ShopModel.Address,
                            DateOpening = form.ShopModel.DateOpening,
                        };

                        var modelFurn = new FurnitureBindingModel
                        {
                            Id = form.FurnitureId,
                            FurnitureName = form.FurnitureModel.FurnitureName,
                            Price = form.FurnitureModel.Price,
                            FurnitureComponents = form.FurnitureModel.FurnitureComponents
                        };

                        var operationResult = _shopLogic.AddFurniture(modelShop, modelFurn, form.Count);
                        if (!operationResult)
                        {
                            throw new Exception("Ошибка при пополнении магазина.  Дополнительная информация в логах.");
                        }
                    }
                
                }
        }
    }
}
